UNIT StrBloq; {Copyright 1987, 1996 Paul Milenkovic ALL RIGHTS RESERVED}

INTERFACE

TYPE
    ASCIIZ_string     = STRING[128];
    string_block      = ARRAY [0 .. 1023] OF CHAR;
    string_blockP     = ^string_block;
    string_block_file = FILE OF string_block;


PROCEDURE InitStringBlock(VAR block: string_block; n: INTEGER);

FUNCTION  EnterStringBlock(

    VAR block: string_block; p, n: INTEGER; src: STRING

): INTEGER;

FUNCTION  RetrieveStringBlock(

    VAR dst: STRING; VAR block: string_block; p, n: INTEGER

): INTEGER;

FUNCTION  RetrieveStringBlockSuffix(

    VAR count: INTEGER; VAR block: string_block; p, n: INTEGER

): INTEGER;

FUNCTION  EnvPtr: string_blockP;

FUNCTION  ComSpec: ASCIIZ_string;


FUNCTION  CharPosString(VAR str: ASCIIZ_string; i: INTEGER; ch: CHAR): INTEGER;

PROCEDURE RemoveLeadingSpaces(VAR str: ASCIIZ_string);

PROCEDURE MakeUpCase(VAR str: ASCIIZ_string);


FUNCTION  StringKeyword(

    VAR dest: ASCIIZ_string; VAR src: STRING; key: STRING

): BOOLEAN;


FUNCTION  IntegerKeyword(

    VAR dest: INTEGER; VAR src: STRING; key: STRING

): BOOLEAN;


FUNCTION  WordHexKeyword(

    VAR dest: WORD; VAR src: STRING; key: STRING

): BOOLEAN;


FUNCTION  SingleKeyword(

    VAR dest: SINGLE; VAR src: STRING; key: STRING

): BOOLEAN;


FUNCTION  DoubleKeyword(

    VAR dest: DOUBLE; VAR src: STRING; key: STRING

): BOOLEAN;


FUNCTION  HexString(n: WORD; m: INTEGER): ASCIIZ_string;

FUNCTION  IntString(n, m: INTEGER): ASCIIZ_string;


FUNCTION  FileExists(fname: ASCIIZ_string): BOOLEAN;

FUNCTION  FileReadOnly(fname: ASCIIZ_string): BOOLEAN;

PROCEDURE EraseFile(fname: ASCIIZ_string);

FUNCTION  OpenTextInput (VAR tfile: TEXT; tname: ASCIIZ_string): BOOLEAN;

FUNCTION  OpenTextOutput(VAR tfile: TEXT; tname: ASCIIZ_string): BOOLEAN;


FUNCTION  RootPath  (path: ASCIIZ_string): ASCIIZ_string;

FUNCTION  PathExt   (path: ASCIIZ_string): ASCIIZ_string;

FUNCTION  FileInPath(path: ASCIIZ_string): ASCIIZ_string;

FUNCTION  DirInPath (path: ASCIIZ_string): ASCIIZ_string;

FUNCTION  NewExt(path, ext: ASCIIZ_string): ASCIIZ_string;


IMPLEMENTATION

PROCEDURE InitStringBlock(VAR block: string_block; n: INTEGER);

BEGIN
    FillChar(block,n,$00);
END;


FUNCTION  EnterStringBlock(

    VAR block: string_block; p, n: INTEGER; src: STRING

): INTEGER;

    VAR i: INTEGER;

BEGIN
    i := 0;
    WHILE (i < Length(src)) AND (Succ(p) < Pred(n)) DO BEGIN
        Inc(i);
        block[p] := src[i];
        Inc(p);
    END;
    IF p < n THEN BEGIN
        block[p] := #00;
        Inc(p);
    END;
    EnterStringBlock := p;
END;


FUNCTION  RetrieveStringBlock(

    VAR dst: STRING; VAR block: string_block; p, n: INTEGER

): INTEGER;

    VAR i: INTEGER;

BEGIN
    i := 0;
    WHILE (p < Pred(n)) AND (block[p] <> #00) DO BEGIN
        Inc(i);
        dst[i] := block[p];
        Inc(p);
    END;
    dst[0] := CHAR(i);
    IF (i > 0) AND (p < Pred(n)) THEN
        Inc(p);
    RetrieveStringBlock := p;
END;


FUNCTION  RetrieveStringBlockSuffix(

    VAR count: INTEGER; VAR block: string_block; p, n: INTEGER

): INTEGER;

    VAR i: INTEGER;

BEGIN
    count := 0;
    IF p < n - 3 THEN BEGIN
        IF block[p] = #0 THEN BEGIN
            i := BYTE(block[p+1]);
            i := i OR (BYTE(block[p+2]) SHL 8);
            IF i > 0 THEN BEGIN
                count := i;
                Inc(p,3);
            END;
        END;
    END;
    RetrieveStringBlockSuffix := p;
END;


FUNCTION  EnvPtr: string_blockP;

    VAR env_ptr: ^string_block;

BEGIN
    EnvPtr := string_blockP(Ptr(INTEGER(Ptr(PrefixSeg,$002C)^),0));
END;


FUNCTION ComSpec: ASCIIZ_string;

    VAR system_path, estr: ASCIIZ_string;
	i, p: INTEGER;

BEGIN
    system_path := '';
    p := 0;
    REPEAT
        p := RetrieveStringBlock(estr,EnvPtr^,p,MaxInt);
        IF StringKeyword(system_path,estr,'COMSPEC=') THEN;
    UNTIL estr = '';
    i := 1;
    WHILE i <= Length(system_path) DO BEGIN
        IF system_path[i] = ';' THEN system_path[0] := CHAR(Pred(i));
        Inc(i);
    END;
    ComSpec := system_path;
END;


FUNCTION  FindChar(VAR str: ASCIIZ_string; i: INTEGER; ch: CHAR): INTEGER;

BEGIN
    WHILE (i <= Length(str)) AND (str[i] <> ch) DO
        Inc(i);
    FindChar := i;
END;


FUNCTION  SkipChar(VAR str: ASCIIZ_string; i: INTEGER; ch: CHAR): INTEGER;

BEGIN
    WHILE (i <= Length(str)) AND (str[i] = ch) DO
        Inc(i);
    SkipChar := i;
END;


FUNCTION  CharPosString(VAR str: ASCIIZ_string; i: INTEGER; ch: CHAR): INTEGER;

BEGIN
    CharPosString := 0;
    WHILE i <= Length(str) DO
        IF str[i] = ch THEN BEGIN
            CharPosString := i;
            i := Succ(Length(str));
        END ELSE
            Inc(i);
END;


PROCEDURE RemoveLeadingSpaces(VAR str: ASCIIZ_string);

    VAR i: INTEGER;

BEGIN
    i := SkipChar(str,1,' ');
    str := Copy(str,i,Succ(Length(str)-i));
END;


PROCEDURE MakeUpCase(VAR str: ASCIIZ_string);

    VAR i: INTEGER;

BEGIN
    FOR i := 1 TO Length(str) DO str[i] := UpCase(str[i]);
END;


FUNCTION  StringKeyword(

    VAR dest: ASCIIZ_string; VAR src: STRING; key: STRING

): BOOLEAN;

    VAR n: INTEGER;

BEGIN
    n := Length(key);
    IF Copy(src,1,n) = key THEN BEGIN
        dest := Copy(src,n+1,Length(src)-n);
        StringKeyword := TRUE;
    END ELSE
        StringKeyword := FALSE;
END;


FUNCTION  IntegerKeyword(

    VAR dest: INTEGER; VAR src: STRING; key: STRING

): BOOLEAN;

    VAR n, err: INTEGER;

BEGIN
    n := Length(key);
    IF Copy(src,1,n) = key THEN BEGIN
        Val(Copy(src,n+1,Length(src)-n),dest,err);
        IntegerKeyword := TRUE;
    END ELSE
        IntegerKeyword := FALSE;
END;


FUNCTION  WordHexKeyword(

    VAR dest: WORD; VAR src: STRING; key: STRING

): BOOLEAN;

    VAR i, n: INTEGER;

BEGIN
    n := Length(key);
    IF Copy(src,1,n) = key THEN BEGIN
        dest := 0;
        FOR i := Succ(n) TO Length(src) DO
            dest := (dest SHL 4) + (BYTE(src[i]) AND $0F) +
                    9*(BYTE(src[i]) SHR 6);
        WordHexKeyword := TRUE;
    END ELSE
        WordHexKeyword := FALSE;
END;


FUNCTION  SingleKeyword(

    VAR dest: SINGLE; VAR src: STRING; key: STRING

): BOOLEAN;

    VAR n, err: INTEGER;

BEGIN
    n := Length(key);
    IF Copy(src,1,n) = key THEN BEGIN
        Val(Copy(src,n+1,Length(src)-n),dest,err);
        SingleKeyword := TRUE;
    END ELSE
        SingleKeyword := FALSE;
END;


FUNCTION  DoubleKeyword(

    VAR dest: DOUBLE; VAR src: STRING; key: STRING

): BOOLEAN;

    VAR n, err: INTEGER;

BEGIN
    n := Length(key);
    IF Copy(src,1,n) = key THEN BEGIN
        Val(Copy(src,n+1,Length(src)-n),dest,err);
        DoubleKeyword := TRUE;
    END ELSE
        DoubleKeyword := FALSE;
END;


FUNCTION  HexString(n: WORD; m: INTEGER): ASCIIZ_string;

    VAR sval: STRING[4];
        i: INTEGER;
        fld: WORD;

BEGIN
    sval := '';
    IF m < 1 THEN m := 1;
    FOR i := 0 TO 3 DO BEGIN
        fld := n SHR 12;
        n   := n SHL 4;
        IF (sval <> '') OR (fld <> 0) OR (m+i > 3) THEN BEGIN
            IF fld > 9 THEN
                sval := Concat(sval,CHAR(55+fld))
            ELSE
                sval := Concat(sval,CHAR(48+fld));
        END;
    END;
    HexString := sval;
END;


FUNCTION  IntString(n, m: INTEGER): ASCIIZ_string;

    VAR sval: ASCIIZ_string;

BEGIN
    Str(n:m,sval);
    IntString := sval;
END;


FUNCTION FileExists(fname: ASCIIZ_string): BOOLEAN;

    VAR tfile: FILE;

BEGIN
    FileExists := FALSE;
    Assign(tfile,fname);
    FileMode := 0;
    {$I-} Reset(tfile) {$I+};
    FileMode := 2;
    IF IOResult = 0 THEN BEGIN
        Close(tfile);
        FileExists := TRUE;
    END;
END;


FUNCTION FileReadOnly(fname: ASCIIZ_string): BOOLEAN;

    VAR tfile: FILE;

BEGIN
    Assign(tfile,fname);
    FileMode := 0;
    {$I-} Reset(tfile) {$I+};
    FileMode := 2;
    IF IOResult = 0 THEN BEGIN
        {$I-} Reset(tfile) {$I+};
        IF IOResult = 0 THEN BEGIN
            Close(tfile);
            FileReadOnly := FALSE;
        END ELSE
            FileReadOnly := TRUE;
    END ELSE BEGIN
        {$I-} Rewrite(tfile) {$I+};
        IF IOResult = 0 THEN BEGIN
            Close(tfile);
            Erase(tfile);
            FileReadOnly := FALSE;
        END ELSE
            FileReadOnly := TRUE;
    END;
END;


PROCEDURE EraseFile(fname: ASCIIZ_string);

    VAR tfile: FILE;

BEGIN
    Assign(tfile,fname);
    {$I-} Erase(tfile) {$I+};
    IF IOResult = 0 THEN;
END;


FUNCTION  OpenTextInput (VAR tfile: TEXT; tname: ASCIIZ_string): BOOLEAN;

BEGIN
    Assign(tfile,tname);
    FileMode := 0;
    {$I-} Reset(tfile) {$I+};
    FileMode := 2;
    IF IOResult = 0 THEN
        OpenTextInput := TRUE
    ELSE
        OpenTextInput := FALSE;
END;


FUNCTION  OpenTextOutput(VAR tfile: TEXT; tname: ASCIIZ_string): BOOLEAN;

BEGIN
    Assign(tfile,tname);
    {$I-} Rewrite(tfile) {$I+};
    IF IOResult = 0 THEN
        OpenTextOutput := TRUE
    ELSE
        OpenTextOutput := FALSE;
END;


FUNCTION  RootPath(path: ASCIIZ_string): ASCIIZ_string;

    VAR p, q: INTEGER;

BEGIN
    p := Length(path);
    WHILE (p > 1) AND ((path[p] = ' ') OR (path[p] = #0)) DO Dec(p);
    q := p;
    WHILE (p > 1) AND (path[p] <> '.') AND (path[p] <> '\') DO Dec(p);
    IF path[p] = '.' THEN RootPath := Copy(path,1,Pred(p))
                     ELSE RootPath := Copy(path,1,q);
END;


FUNCTION  PathExt(path: ASCIIZ_string): ASCIIZ_string;

    VAR p, q: INTEGER;

BEGIN
    p := Length(path);
    WHILE (p > 1) AND ((path[p] = ' ') OR (path[p] = #0)) DO Dec(p);
    q := p;
    WHILE (p > 1) AND (path[p] <> '.') AND (path[p] <> '\') DO Dec(p);
    IF path[p] = '.' THEN PathExt := Copy(path,Succ(p),Length(path)-p)
                     ELSE PathExt := '';
END;


FUNCTION  ParseFile(VAR fname: ASCIIZ_string): INTEGER;

    VAR p: INTEGER;

BEGIN
    ParseFile := 0;
    p := Length(fname);
    WHILE p > 0 DO
        IF (fname[p] = '\') OR (fname[p] = ':') THEN BEGIN
            ParseFile := p;
            p := 0;
        END ELSE
            Dec(p);
END;


FUNCTION  FileInPath(path: ASCIIZ_string): ASCIIZ_string;

    VAR p: INTEGER;

BEGIN
    p := ParseFile(path);
    FileInPath := Copy(path,Succ(p),Length(path)-p);
END;


FUNCTION  DirInPath (path: ASCIIZ_string): ASCIIZ_string;

    VAR p: INTEGER;

BEGIN
    p := ParseFile(path);
    IF p > 1 THEN
        IF (path[p] = '\') AND (path[Pred(p)] <> ':') THEN Dec(p);
    DirInPath := Copy(path,1,p);
END;


FUNCTION  NewExt(path, ext: ASCIIZ_string): ASCIIZ_string;

BEGIN
    IF ext = '' THEN
        NewExt := RootPath(path)
    ELSE
        NewExt := Concat(RootPath(path),'.',ext);
END;

END.
